<?php
namespace youlead\Admin;
use youlead\Helper\Config;

class Admin extends Config
{
    public function __construct()
    {
        add_action('admin_init', array($this,'youlead_settings'));
        add_action('admin_menu', array($this, 'youlead_add_menu'));
        add_action('admin_bar_menu', array($this, 'youlead_toolbar'), 998);
        add_action('admin_post_clear_logs', array($this, 'youlead_clear_logs'));
    }

    public function youlead_toolbar($wp_admin_bar)
    {
        $args = array(
            'id'    => 'youlead',
            'title' => 'YouLead',
            'href'  => 'admin.php?page=youlead-config',
            'meta'  => array(
                'title' => 'YouLead | Sales & Marketing Automation'
            )
        );
        $wp_admin_bar->add_node($args);
    }

    public function youlead_add_menu()
    {
        add_menu_page(
            'YL',
            'YouLead',
            'manage_options',
            'youlead-config',
            array($this, 'youleadConfiguration'),
            plugin_dir_url(__FILE__) . 'images/youlead-icon.png',
            10
        );
    }

    public function youleadConfiguration()
    {
        ?>
            <h1>YouLead - konfiguracja</h1> <br>
            <form method="post" action="options.php">
                <?php 
                    settings_fields('youlead_settings');
                    do_settings_sections('youlead_settings');
                ?>
                <table class="form-table">
                    <tr>
                        <th scope="row">
                            <label for="YLclientId">Client ID:</label>
                        </th>
                        <td>
                            <input id="YLclientId" type="text" class="regular-text" name="<?php echo self::ClientId ?>" 
                            value="<?php echo esc_attr(get_option(self::ClientId)); ?>" />
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="YLappId">AppId:</label>
                        </th>
                        <td>
                            <input id="YLappId" type="text" class="regular-text" name="<?php echo self::AppId ?>" 
                            value="<?php echo esc_attr(get_option(self::AppId)); ?>" />
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="YLsecret">AppSecretKey:</label>
                        </th>
                        <td>
                            <input id="YLsecret" type="text" class="regular-text" name="<?php echo self::SecretKey ?>"
                            value="<?php echo esc_attr(get_option(self::SecretKey)); ?>" />
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="YLAttrMap">Mapowanie atrybutów*:</label>
                        </th>
                        <td>
                            <textarea id="YLAttrMap" class="regular-text" rows="6" name="<?php echo self::YLAttrMap ?>"><?php echo esc_attr(get_option(self::YLAttrMap)); ?></textarea>
                            <p class="description">
                                Atrybuty powinny być podane jako para <b>klucz:wartość</b>, gdzie kluczem jest nazwa atrybutu w formularzu, a wartością jego odpowiednik w YouLead.<br>
                                Każda para atrybutów powinna być oddzielona średnikiem.<br>Przykład:<br>
                                <i>imie:name;<br>mail:email;</i>
                            </p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"></th>
                        <td>
                            <label>
                            <?php
                                $YlLoggingEnabled = self::YlLoggingEnabled;
                                $checked = esc_attr(get_option($YlLoggingEnabled)) ? 'checked' : '';

                                echo "<input type='checkbox' name='$YlLoggingEnabled' $checked /> Włącz logi*";
                            ?>
                            </label>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">Sterowanie śledzeniem za pomoicą ciastka cookie bota</th>
                        <td>
                            <label>
                            <?php
                                $YlBlockingTrackCookie = self::YlBlockingTrackCookie;
                                $checked = esc_attr(get_option($YlBlockingTrackCookie)) ? 'checked' : '';

                                echo "<input type='checkbox' name='$YlBlockingTrackCookie' $checked /> Dodaj ciastko blokowania śledzenia*";
                            ?>
                            </label>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="YLCookieBotName">Nazwa ciastka cookie bota:</label>
                        </th>
                        <td>
                            <input id="YLCookieBotName" type="text" class="regular-text" name="<?php echo self::YLCookieBotName ?>"
                            value="<?php echo esc_attr(get_option(self::YLCookieBotName)); ?>" />
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="YLCookieBotSeparator">Znak oddzielający wartości w ciastku cookie bota np.: "," ";" <br>Domyślnie jest to przecinek ","</label>
                        </th>
                        <td>
                            <input id="YLCookieBotSeparator" type="text" class="regular-text" name="<?php echo self::YLCookieBotSeparator ?>"
                            value="<?php echo esc_attr(get_option(self::YLCookieBotSeparator)); ?>" />
                        </td>
                    </tr>
                    <tr>
                        <th scope="row">
                            <label for="YLCookieBotAttrMap">Wymagane wartości w ciastku do włączenia śledzenia:</label>
                        </th>
                        <td>
                            <textarea id="YLCookieBotAttrMap" class="regular-text" rows="6" name="<?php echo self::YLCookieBotAttrMap ?>"><?php echo esc_attr(get_option(self::YLCookieBotAttrMap)); ?></textarea>
                            <p class="description">
                                Zgody powinny być rozpisane jako para <b>klucz:wartość</b>. Klucz to nazwa zgody w ciastku cookie bota. Wartość to wyrażona zgoda dla danego ciastka.<br>
                                Każda para zgód powinna być oddzielana średnikiem.<br>Przykładowy zestaw par klucz:wartość:<br>
                                <i>advertisement:yes;<br>marketing:true;</i>
                            </p>
                        </td>
                    </tr>
                </table>
                <i>* Pole opcjonalne</i>
                <?php submit_button(); ?>
            </form>
            <?php
            if (file_exists(self::LogPath)) {
                echo '<a href="'. content_url(self::LogName) .'" download class="button button-primary">Pobierz plik z logami</a>
                <form method="post" action="' . admin_url('admin-post.php') . '">
                <input type="hidden" name="action" value="clear_logs">';
                submit_button('Wyczyść logi');
                echo '</form>';
            }
            echo '<script>var ylCookieElem = document.querySelectorAll("[name^=\'yl_cookie\']");
            var checked = document.querySelector("[name=\'yl_blocking_track_cookie_enabled\']");
            
              ylCookieElem.forEach(function(el){
                if(!checked.checked)
                {
                    el.setAttribute("disabled", "disabled");
                }
                else
                {
                    el.removeAttribute("disabled");
                }
              });
            
            checked.addEventListener(\'click\', function(){
              ylCookieElem.forEach(function(el){
                if(!checked.checked)
                {
                    el.setAttribute("disabled", "disabled");
                }
                else
                {
                    el.removeAttribute("disabled");
                }
              });
            });</script>';

    }

    public function youlead_clear_logs()
    {
        if (file_exists(self::LogPath)) {
            unlink(self::LogPath);
        }

        $url = 'admin.php?page=youlead-config';
        wp_safe_redirect($url);
        exit;
    }

    public function youlead_settings()
    {
        register_setting('youlead_settings', self::ClientId);
        register_setting('youlead_settings', self::AppId);
        register_setting('youlead_settings', self::SecretKey);
        register_setting('youlead_settings', self::YlLoggingEnabled);
        register_setting('youlead_settings', self::YLAttrMap);
        register_setting('youlead_settings', self::YlBlockingTrackCookie);
        register_setting('youlead_settings', self::YLCookieBotName);
        register_setting('youlead_settings', self::YLCookieBotAttrMap);
        register_setting('youlead_settings', self::YLCookieBotSeparator);
    }
}