<?php
class Loader
{
    private $prefix;
    private $directory;
    private $valid = false;

    public function __construct($prefix, $directory)
    {
        $this->prefix = (string)$prefix;

        if (!is_dir($directory)) {
            throw new RuntimeException($directory.' is not a valid directory path.');
        }
        
        $this->directory = $directory;
        $this->valid = true;
    }

    public function autoload($className)
    {

        if (strpos($className, $this->prefix) !== 0) {
            return false;
        }

        $className = str_replace($this->prefix, '', $className);
        require_once($this->directory . $className . '.php');
        return true;
    }

    public function register()
    {
        if ($this->valid) {
            spl_autoload_register(array($this, 'autoload'));
            $this->valid = false;
        }
    }
}

$adminLoader = new Loader('youlead\Admin\\', __DIR__ . '/admin/');
$includesLoader = new Loader('youlead\Tracking\\',  __DIR__ . '/includes/');
$otherLoader = new Loader('youlead\Helper\\',  __DIR__ . '/helper/');

$adminLoader->register();
$includesLoader->register();
$otherLoader->register();

