<?php
namespace youlead\Helper;
use youlead\Tracking\Logger;

class YLApi extends Config
{
    private $clientId;
    private $appId;
    private $appSk;
    private $attrMap;

    public function __construct()
    {
        $this->init();
    }

    private function init()
    {
        $this->clientId = get_option(self::ClientId);
        $this->appId = get_option(self::AppId);
        $this->appSk = get_option(self::SecretKey);
        $attrMap = get_option(self::YLAttrMap);

        $attrMap = preg_replace('/\s+/', '', $attrMap);
        $attributes = explode(';', $attrMap);

        $this->attrMap = [];

        for ($i = 0; $i < count($attributes); $i++) {
            $attr = explode(':', $attributes[$i]);
            if (count($attr) > 1) {
                if (!array_key_exists($attr[0], $this->attrMap)) {
                    $this->attrMap[$attr[0]] = $attr[1];
                }
            }
        }
    }

    public function FillForm($formId, $formData, $formUrl = '')
    {
        global $wp;
        $currentUrl = home_url(add_query_arg(array(), $wp->request));
        $ylidCookie = isset($_COOKIE['ylid']) ? $_COOKIE['ylid'] : '';

        $parsedFormData = $this->getFormData($formData);

        if (empty($parsedFormData))
            return;

        if (!empty($parsedFormData['cid'])) {
           $ylidCookie = $parsedFormData['cid'];
        }

        if (!empty($parsedFormData['ylid'])) {
            $ylidCookie = $parsedFormData['ylid'];
        }

        $dataArray = array(
            'ylid' => $ylidCookie,
            'pageUrl' => $formUrl != '' ? urlencode($formUrl) : urlencode($currentUrl),
            'formId' => 'wpcf7_' . $formId, 
            'formData' => $parsedFormData
        );

        if (isset($_COOKIE['ylssid'])) {
            $dataArray['sessionId'] = $_COOKIE['ylssid'];
        }

        if (isset($_COOKIE['ylutm'])) {
            $dataArray['utm'] = $_COOKIE['ylutm'];
        }

        $response = $this->sendRequest('Contact/FillForm', $dataArray);
        $logger = new Logger();
        $logMsg = 'YouLead FillForm. Data:' . PHP_EOL . print_r($dataArray, true) . PHP_EOL . 'Response:' . print_r($response, true);
        $logger->add($logMsg);

        try {
            $data = json_decode($response);

            if (isset($data->returnData)) {
                if (isset($data->returnData->ylid)) {
                    $ylid = $data->returnData->ylid;
                    if (!empty($ylid)) {
                        if ($_COOKIE['ylid'] != 'browserId=' . $ylid) {
                            $cookieValue = "browserId=$ylid";
                            $domain = $this->getDomainName($_SERVER['HTTP_HOST']);
                            $cookieTime = time() + 86400 * 365 * 10;

                            $sessionCookieTime = time() + 11 * 60 * 1000;
                            $sessionId = $this->generateGuid();
                            $sessionValue = "sessionId=$sessionId";

                            setrawcookie('ylid', $cookieValue, $cookieTime, '/', '.' . $domain);
                            setrawcookie('ylssid', $sessionValue, $sessionCookieTime, '/', '.' . $domain);
                            setrawcookie('ylsetid', $cookieValue, $cookieTime, '/', '.' . $domain);
                        }
                    }
                }
            }
        } catch (\Exception $ex) {
            $logMsg = 'Failed to convert response' . PHP_EOL . $ex;
            $logger->add($logMsg);
        }
        
        if (isset($_COOKIE['ylDebug']))
            return $response;
    }

    public function TestEcho()
    {
        return $this->sendRequest('Test/Echo');
    }

    private function sendRequest($path, $params)
    {
        $signature = SHA1($this->clientId . $this->appId . $this->appSk . time());

        $headers = array(
            'Content-Type: application/x-www-form-urlencoded',
            'YL-ClientId: '. $this->clientId,
            'YL-AppId: '. $this->appId,
            'YL-TimeStamp: '. time(),
            'YL-Signature: '. $signature
        );

        $requestUrl = 'https://a-' . $this->clientId . '.youlead.pl/api/Command/' . $path;
        $curl = curl_init($requestUrl);
        curl_setopt($curl, CURLOPT_POSTFIELDS, 'data=' . json_encode($params));
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_TIMEOUT_MS, 60000);
        curl_setopt($curl, CURLOPT_NOSIGNAL, 1);
        $response = curl_exec($curl);

        curl_close($curl);

        return $response;
    }

    private function getFormData($data)
    {
        $formData = array();
        foreach ($data as $key => $value) {
            if (!in_array($key, $this->blackListParams())) {
                $key = str_replace('yl-', '', $key);
				$keys = explode(':', $key);
				
				foreach ($keys as $key) {
                    // check array with mapped attributes
                    if (array_key_exists($key, $this->attrMap)) {
                        $key = $this->attrMap[$key];
                    }

					if (is_array($value)) {
						if(count($value) === 1)
						{
							try {
							    foreach ($value as $k => $v) {
								    if ($v)
									    $formData[$key] = $this->parseFormData($v);
							    }
							} catch (Exception $e) {
							}
						}
						else 
						{
							try {
							    foreach ($value as $k => $v) {
								    if ($v)
									    $formData[$key . '_' . $k] = $this->parseFormData($v);
							    }
							} catch (Exception $e) {
							}
						}						
					} else {
						if ($value)
							$formData[$key] = $this->parseFormData($value);
					}
				}
            }
        }

        return $formData;
    }

    private function blackListParams()
    {
        $blackList = array('_wpcf7', '_wpcf7_version', '_wpcf7_locale', '_wpcf7_unit_tag', '_wpcf7_container_post',
            'g-recaptcha-response', 'password', 'password_confirmation', 'password_confirm', 'haslo',
            'haslo_potwierdzenie');

        return $blackList;
    }

    private function parseFormData($v)
    {
        if ($v == 'on') {
            $v = 1;
        } elseif ($v == 'off') {
            $v = 0;
        }
        //return addslashes(stripslashes(strtr($v, ['&' => '%26'])));
        return strtr($v, ['&' => '%26']);
    }

    private function getDomainName($url)
    {
        $TLDs = ['lubin', 'warszawa', 'katowice', 'wroclaw', 'lublin', 'gda', 'waw', 'local', 'ac', 'ad', 'ae', 'aero', 'af', 'ag', 'ai', 'al', 'am', 'an', 'ao', 'aq', 'ar', 'arpa', 'as', 'asia', 'at', 'au', 'aw', 'ax', 'az', 'ba', 'bb', 'bd', 'be', 'bf', 'bg', 'bh', 'bi', 'biz', 'bj', 'bm', 'bn', 'bo', 'br', 'bs', 'bt', 'bv', 'bw', 'by', 'bz', 'ca', 'cat', 'cc', 'cd', 'cf', 'cg', 'ch', 'ci', 'ck', 'cl', 'cm', 'cn', 'co', 'com', 'coop', 'cr', 'cu', 'cv', 'cx', 'cy', 'cz', 'de', 'dj', 'dk', 'dm', 'do', 'dz', 'ec', 'edu', 'ee', 'eg', 'er', 'es', 'et', 'eu', 'fi', 'fj', 'fk', 'fm', 'fo', 'fr', 'ga', 'gb', 'gd', 'ge', 'gf', 'gg', 'gh', 'gi', 'gl', 'gm', 'gn', 'gov', 'gp', 'gq', 'gr', 'gs', 'gt', 'gu', 'gw', 'gy', 'hk', 'hm', 'hn', 'hr', 'ht', 'hu', 'id', 'ie', 'il', 'im', 'in', 'info', 'int', 'io', 'iq', 'ir', 'is', 'it', 'je', 'jm', 'jo', 'jobs', 'jp', 'ke', 'kg', 'kh', 'ki', 'km', 'kn', 'kp', 'kr', 'kw', 'ky', 'kz', 'la', 'lb', 'lc', 'li', 'lk', 'lr', 'ls', 'lt', 'lu', 'lv', 'ly', 'ma', 'mc', 'md', 'me', 'mg', 'mh', 'mil', 'mk', 'ml', 'mm', 'mn', 'mo', 'mobi', 'mp', 'mq', 'mr', 'ms', 'mt', 'mu', 'museum', 'mv', 'mw', 'mx', 'my', 'mz', 'na', 'name', 'nc', 'ne', 'net', 'nf', 'ng', 'ni', 'nl', 'no', 'np', 'nr', 'nu', 'nz', 'om', 'org', 'pa', 'pe', 'pf', 'pg', 'ph', 'pk', 'pl', 'pm', 'pn', 'pr', 'pro', 'ps', 'pt', 'pw', 'py', 'qa', 're', 'ro', 'rs', 'ru', 'rw', 'sa', 'sb', 'sc', 'sd', 'se', 'sg', 'sh', 'shop', 'si', 'sj', 'sk', 'sl', 'sm', 'sn', 'so', 'sr', 'st', 'su', 'sv', 'sy', 'sz', 'tc', 'td', 'tel', 'tf', 'tg', 'th', 'tj', 'tk', 'tl', 'tm', 'tn', 'to', 'tp', 'tr', 'travel', 'tt', 'tv', 'tw', 'tz', 'ua', 'ug', 'uk', 'us', 'uy', 'uz', 'va', 'vc', 've', 'vg', 'vi', 'vn', 'vu', 'wf', 'ws', 'xn--0zwm56d', 'xn--11b5bs3a9aj6g', 'xn--3e0b707e', 'xn--45brj9c', 'xn--80akhbyknj4f', 'xn--90a3ac', 'xn--9t4b11yi5a', 'xn--clchc0ea0b2g2a9gcd', 'xn--deba0ad', 'xn--fiqs8s', 'xn--fiqz9s', 'xn--fpcrj9c3d', 'xn--fzc2c9e2c', 'xn--g6w251d', 'xn--gecrj9c', 'xn--h2brj9c', 'xn--hgbk6aj7f53bba', 'xn--hlcj6aya9esc7a', 'xn--j6w193g', 'xn--jxalpdlp', 'xn--kgbechtv', 'xn--kprw13d', 'xn--kpry57d', 'xn--lgbbat1ad8j', 'xn--mgbaam7a8h', 'xn--mgbayh7gpa', 'xn--mgbbh1a71e', 'xn--mgbc0a9azcg', 'xn--mgberp4a5d4ar', 'xn--o3cw4h', 'xn--ogbpf8fl', 'xn--p1ai', 'xn--pgbs0dh', 'xn--s9brj9c', 'xn--wgbh1c', 'xn--wgbl6a', 'xn--xkc2al3hye2a', 'xn--xkc2dl3a5ee0h', 'xn--yfro4i67o', 'xn--ygbi2ammx', 'xn--zckzah', 'xxx', 'ye', 'yt', 'za', 'zm', 'zw'];
        $parts = explode('.', $url);
        $ln = count($parts);
        $i = $ln;
        $minLength = strlen($parts[count($parts) - 1]);
        $part = '';
        $tail = '';

        while ($part = $parts[--$i]) {
            if (!in_array($part,$TLDs) || strlen($part) < $minLength || $i < $ln - 2 || $i == 0) {
                return $part . $tail;
            }
            $tail = '.' . $part . $tail;
        }
    }

    private function generateGuid()
    {
        if (function_exists('com_create_guid') === true)
        {
            return trim(com_create_guid(), '{}');
        }

        return strtolower(sprintf('%04X%04X-%04X-%04X-%04X-%04X%04X%04X', 
            random_int(0, 65535), random_int(0, 65535), random_int(0, 65535), random_int(16384, 20479), 
            random_int(32768, 49151), random_int(0, 65535), random_int(0, 65535), random_int(0, 65535)));
    }
}
